package com.wdkl.ncs.android.middleware.tcp

import android.util.Log
import com.enation.javashop.net.engine.utils.ThreadFromUtils
import io.netty.bootstrap.Bootstrap
import io.netty.channel.Channel
import io.netty.channel.ChannelInitializer
import io.netty.channel.ChannelOption
import io.netty.channel.nio.NioEventLoopGroup
import io.netty.channel.socket.SocketChannel
import io.netty.channel.socket.nio.NioSocketChannel
import io.netty.handler.codec.LengthFieldBasedFrameDecoder
import io.netty.handler.codec.LengthFieldPrepender
import io.netty.handler.codec.string.StringDecoder
import io.netty.handler.codec.string.StringEncoder
import io.netty.handler.timeout.IdleStateHandler
import io.netty.util.CharsetUtil
import io.reactivex.Observable
import java.util.concurrent.Executors
import java.util.concurrent.ScheduledExecutorService
import java.util.concurrent.TimeUnit
import kotlin.properties.Delegates

class NettyClient{
    private val TAG = NettyClient::class.java.simpleName

    /*是否连接*/
    private var isConnect = false

    /*伴生对象*/
    companion object {
        var instance = NettyClient()
        var isConnecting = false

    }

    private lateinit var group: NioEventLoopGroup
    private lateinit var bootstrap: Bootstrap
    private var channel: Channel? = null

    //重连间隔
    private val retrySeconds : Long = 2

    //重连计数
    private var retryTimes = 0

    private lateinit var ip: String
    private var port by Delegates.notNull<Int>()
    private var heartBeatSeconds by Delegates.notNull<Long>()

    //数据处理
    private var tcpClientHandler = TcpClientHandler()

    fun getChannel(): Channel? {
        return channel
    }

    /*连接*/
    @Synchronized
    fun connect(ip: String, port: Int, heartBeatSeconds: Long): Observable<Boolean> {
        Log.d(TAG, "开始连接 " + ip + ":" + port +","+heartBeatSeconds)
        isConnecting = true

        this.ip = ip
        this.port = port
        this.heartBeatSeconds = heartBeatSeconds

        return Observable.create<Boolean> {
            group = NioEventLoopGroup()
            bootstrap = Bootstrap()
                    .remoteAddress(ip, port)
                    .group(group)
                    .channel(NioSocketChannel::class.java)
                    .option(ChannelOption.SO_KEEPALIVE, true)
                    .option(ChannelOption.CONNECT_TIMEOUT_MILLIS, 3000)
                    .handler(object : ChannelInitializer<SocketChannel>() {
                        override fun initChannel(sc: SocketChannel) {
                            val pipeline = sc.pipeline()

                            // 这里将LengthFieldBasedFrameDecoder添加到pipeline的首位，因为其需要对接收到的数据
                            // 进行长度字段解码，这里也会对数据进行粘包和拆包处理
                            pipeline.addLast(LengthFieldBasedFrameDecoder(2048, 0, 2, 0, 2))
                            // LengthFieldPrepender是一个编码器，主要是在响应字节数据前面添加字节长度字段
                            pipeline.addLast(LengthFieldPrepender(2))
                            //心跳包应当小于服务器间隔
                            pipeline.addLast(IdleStateHandler(heartBeatSeconds * 2, heartBeatSeconds, 0, TimeUnit.SECONDS))
                            pipeline.addLast(StringDecoder(CharsetUtil.UTF_8))
                            pipeline.addLast(StringEncoder(CharsetUtil.UTF_8))
                            pipeline.addLast(tcpClientHandler)
                        }
                    })


            try {
                channel = bootstrap.connect().sync().channel()
                it.onNext(channel!!.isActive)
                isConnect = channel!!.isActive
            } catch (e: Exception) {
                isConnect = false
                Log.d(TAG, "TCP.exception：${e.message}")

                if (retryTimes>30){
                    retryTimes = 0
                    it.onNext(false)
                } else {
                    group.schedule(Runnable {
                        group.shutdownGracefully()
                        connect(ip, port, heartBeatSeconds)
                        retryTimes++
                    }, retrySeconds * retryTimes, TimeUnit.SECONDS)
                }
            } finally {
                isConnecting = false
            }
        }.compose(ThreadFromUtils.defaultSchedulers())
    }


    /*发送命令*/
    fun sendMsg(msg: String): Observable<Boolean> {
        Log.d(TAG, "TCP.发送消息：$msg")
        return Observable.create<Boolean> { emitter ->
            if (isConnect) {
                channel?.writeAndFlush(msg)?.addListener {
                    emitter.onNext(it.isSuccess)
                }
            } else {
                emitter.onNext(false)
            }
        }.compose(ThreadFromUtils.defaultSchedulers())
    }

    /*是否连接*/
    fun isConnect(): Boolean {
        return isConnect
    }

    /*重连*/
    @Synchronized
    fun reConnect(): Observable<Boolean> {
        if (isConnecting){
            Log.w(TAG, "TCP.在连接过程中")
            return Observable.create<Boolean>{
                it.onComplete()
            }
        }

        if (channel!=null && channel!!.isActive){
            Log.w(TAG, "TCP.已经连接上")
            return Observable.create<Boolean>{
                it.onComplete()
            }
        }

        isConnecting = true
        return Observable.create<Boolean> {
            try {
                channel = null
                channel = bootstrap.connect().sync().channel();
                it.onNext(channel!!.isActive)
                isConnect = channel!!.isActive
            } catch (e:Exception){
                isConnect = false
                it.onNext(false)
            } finally {
                isConnecting = false
            }
        }

//        disConnect()
//        return connect(ip, port, heartBeatSeconds)
    }


    /*关闭连接*/
    fun disConnect() {
        if (channel!=null) {
            isConnect = false
            group.shutdownGracefully()
        }
    }
}